#/usr/bin/perl 
use POSIX;

$blankline = 0;
#first, read whole grammar to get parents of each non-terminal
#skip intro (up to first \n\n)
while($line = <>)
{   
	chomp $line;
    $line =~ tr/\r//d;
    if($line)
    {
        $blankline = 0;
    }
    else
    {
        last if $blankline;
        $blankline = 1;
    }
}
while ($line = <>)
{
    chomp $line;
    $line =~ tr/\r//d;
    if($line =~ m/^([a-z0-9\-]+) (<\d+>)? \s = \s /x) #rule start
    {
        $non_term = $1;
        $rule_remainder = $';
        add_parents($non_term, $rule_remainder);
    } 
    elsif ($line =~ m/^\s/) #rule continuation
    {
        add_parents($non_term, $line);
    }
}

seek STDIN, 0, 0;
#from the top!
$blankline = 0;
$line = <>;
chomp $line;
$intro = $line . "<br>Converted by Nick Nicholas, nicholas\@uci.edu, " .
    POSIX::strftime("%Y-%m-%d", gmtime(time)) .
    ".<br>.i te galfi fi tu'a la nik. NIkolas. (goi nitcion.) noi se samymrilyjudri \"zoixy. nicholas\@uci.edu \"xy. de'i li " .
    POSIX::strftime("%Ypi'e%mpi'e%d", gmtime(time)) .
    "<hr><b>Lojban<b> - the Logical Language<br>la lojban. po'u le logji bangu<br>http:\/\/www.lojban.org";
$intro =~ tr/\r//d;
print " INFO\t$intro\n";
$intro = "";
<>;
    
#Intro prose up to \n\n
while($line = <>)
{
    chomp $line;
    $line =~ tr/\r//d;
    if($line)
    {   
        $blankline = 0;
    	$line =~ s/</&lt;/g;
    	$line =~ s/>/&gt;/g;
        $intro .= $line . " "    
    }
    else
    {
        last if $blankline;
        $blankline = 1;
        $intro .= "<p>";
    }
}

$intro =~ s/\"stag\"/\"<a>stag<\/a>\"/xg;
$intro =~ s/free/<a>free<\/a>/xg;
$intro =~ s/<p> \s* 1\) \s*/<ol><li>/xg;
$intro =~ s/\s+ \d{1,2} \) \s*/<li>/xg; # don't snarf phone number
$intro =~ s/<p>\t (name .+?) <p>/<ul><li>$1<\/ul>/xg;
$intro =~ s/\t/ /g;
$intro =~ s/  / /g;
print " INTRO\t$intro\n";
print " START\t<i>Parsing starts with </i> <a>text</a>.\n";

#now the grammar
while ($line = <>)
{
    chomp $line;
    $line =~ tr/\r//d;
    if($line =~ m/^([a-z0-9\-]+) (<\d+>)? \s = \s /x) #rule start
    {
        $rule_prefix = $&;
        $non_term = $1;
        $rule_remainder = $';
        $rule_remainder = rule_link($rule_remainder)
        	unless ($rule_remainder =~ m/\"/);
        $rule = $rule_prefix . $rule_remainder
    } 
    elsif ($line =~ m/^\s/) #rule continuation
    {
        $rule .= rule_link($line);
    }
    elsif ($line =~ m/The following rules are non\-formal/)
    {
        $non_formal = 1;
    }
    elsif (not $line) #rule end
    {
    	my @arr =  @{$parent{$non_term}};
    	@arr =  sort @arr;
    	for($i=0; $i< $#arr; $i++)  # elim redundancies
    	{
    	    splice (@arr, $i-- +1, 1) if($arr[$i] eq $arr[$i+1]);
    	}
        printf "%s\t%s%s<hr>%s: ", $non_term, $rule,
            ($non_formal? " (<i>Non-formal rule</i>)" : ""),
            (scalar @arr == 1 ? "Parent" : "Parents");
        printf "<a>%s<\/a>\n", join "</a>, <a>", (@arr);
    }  
    else #error
    {   print STDERR "$line\n";    }
}


sub rule_link        # hyperlink to non-terminals
{
    local $_ = $_[0];
    s/([a-z0-9\-]+) (?![A-Z]) /<a>$1<\/a>/xg; # non-terminal: musn't be followed by capital (which makes it a Lojban letter)
    s/#/<a:free>#<\/a>/g;
    return $_;
}

sub add_parents ($$)    # record parents of non-terminals
{
    (my $non_term, local $_) = @_;
    while(m/(\# | [a-z0-9\-]+) (?![A-Z])/xg)
    {
        push @{$parent{($1 eq "#") ? "free" : $1}}, $non_term;
    }
}